@@ -5,26 +5,39 @@ Module for all things related to lookup tables
 """
 
 import csv
+import datetime as dt
+import json
 import os
 import re
-import json
-from typing import DefaultDict, List, Pattern, Union
-from collections import defaultdict, OrderedDict
-from itertools import chain
-from pathlib import Path
-from operator import methodcaller
+from collections import OrderedDict, defaultdict
 from copy import deepcopy
+from itertools import chain
+from operator import methodcaller
+from pathlib import Path
+from typing import DefaultDict, List, Pattern, Union
 
 import yaml
-import datetime as dt
 
 from g2p import exceptions
-from g2p.mappings.langs import __file__ as LANGS_FILE, LANGS, MAPPINGS_AVAILABLE
-from g2p.mappings.utils import create_fixed_width_lookbehind, escape_special_characters, normalize
-from g2p.mappings.utils import find_mapping, flatten_abbreviations, IndentDumper, load_abbreviations_from_file
-from g2p.mappings.utils import load_from_file, load_mapping_from_path, unicode_escape, validate
-from g2p.mappings.utils import is_dummy, is_ipa, is_xsampa
 from g2p.log import LOGGER
+from g2p.mappings.langs import LANGS, MAPPINGS_AVAILABLE
+from g2p.mappings.langs import __file__ as LANGS_FILE
+from g2p.mappings.utils import (
+    IndentDumper,
+    create_fixed_width_lookbehind,
+    escape_special_characters,
+    find_mapping,
+    flatten_abbreviations,
+    is_dummy,
+    is_ipa,
+    is_xsampa,
+    load_abbreviations_from_file,
+    load_from_file,
+    load_mapping_from_path,
+    normalize,
+    unicode_escape,
+    validate,
+)
 
 GEN_DIR = os.path.join(os.path.dirname(LANGS_FILE), 'generated')
 
@@ -32,9 +45,14 @@ class Mapping():
     """ Class for lookup tables
 
         @param as_is: bool = True
-            Evaluate g2p rules in mapping in the order they are.
+            Affects whether or not rules are sorted or left as is.
+            Please use ``rule_ordering`` instead.
+            If True, Evaluate g2p rules in mapping in the order they are written.
             If False, rules will be reverse sorted by length.
 
+            .. deprecated:: 0.6
+                use ``rule_ordering`` instead
+
         @param case_sensitive: bool = True
             Lower all rules and conversion input
 
@@ -50,6 +68,18 @@ class Mapping():
         @param reverse: bool = False
             Reverse all mappings
 
+        @param rule_ordering: str = "as-written"
+            Affects in what order the rules are applied.
+
+            If set to ``"as-written"``, rules are applied from top-to-bottom in the order that they
+            are written in the source file
+            (previously this was accomplished with ``as_is=True``).
+
+            If set to ``"apply-longest-first"``, rules are first sorted such that rules with the longest
+            input are applied first. Sorting the rules like this prevents shorter rules
+            from taking part in feeding relations
+            (previously this was accomplished with ``as_is=False``).
+
         @param prevent_feeding: bool = False
             Converts each rule into an intermediary form
 
@@ -57,9 +87,10 @@ class Mapping():
 
     def __init__(self, mapping=None, abbreviations: Union[str, DefaultDict[str, List[str]]] = False, **kwargs):
         # should these just be explicit instead of kwargs...
+        # yes, they should
         self.allowable_kwargs = ['language_name', 'display_name', 'mapping', 'in_lang',
-                                 'out_lang', 'out_delimiter', 'as_is', 'case_sensitive',
-                                 'escape_special', 'norm_form', 'prevent_feeding', 'reverse']
+                                 'out_lang', 'out_delimiter', 'as_is', 'case_sensitive', 'rule_ordering',
+                                 'escape_special', 'norm_form', 'prevent_feeding', 'reverse', 'type']
         self.kwargs = OrderedDict(kwargs)
         self.processed = False
         if isinstance(abbreviations, defaultdict) or not abbreviations:
@@ -69,12 +100,12 @@ class Mapping():
                 abbreviations)
         # Handle user-supplied list
         if isinstance(mapping, list):
-            self.mapping = validate(mapping)
+            self.mapping = validate(mapping, path="user-supplied mapping")
         elif isinstance(mapping, str) and (mapping.endswith('yaml') or mapping.endswith('yml')):
             loaded_config = load_mapping_from_path(mapping)
             self.process_loaded_config(loaded_config)
         elif isinstance(mapping, str):
-            self.mapping = validate(load_from_file(mapping))
+            self.mapping = validate(load_from_file(mapping), path=mapping)
         else:
             if "in_lang" in self.kwargs and "out_lang" in self.kwargs:
                 loaded_config = find_mapping(
@@ -83,6 +114,8 @@ class Mapping():
             elif 'id' in self.kwargs:
                 loaded_config = self.find_mapping_by_id(self.kwargs['id'])
                 self.process_loaded_config(loaded_config)
+            elif self.kwargs.get("type", "") == "unidecode":
+                self.mapping = []
             else:
                 raise exceptions.MalformedLookup()
         if self.abbreviations:
@@ -125,6 +158,7 @@ class Mapping():
         for mapping in MAPPINGS_AVAILABLE:
             if mapping.get('id', '') == map_id:
                 return deepcopy(mapping)
+
     @staticmethod
     def mapping_type(name):
         if is_ipa(name):
@@ -164,10 +198,13 @@ class Mapping():
         ''' For a mapping loaded from a file, take the keyword arguments and supply them to the
             Mapping, and get any abbreviations data.
         '''
-        self.mapping = config['mapping_data']
+        if config.get("type", "") == "unidecode":
+            self.mapping = []
+        else:
+            self.mapping = config['mapping_data']
+            self.abbreviations = config.get('abbreviations_data', None)
         mapping_kwargs = OrderedDict(
             {k: v for k, v in config.items() if k in self.allowable_kwargs})
-        self.abbreviations = config.get('abbreviations_data', None)
         # Merge kwargs, but prioritize kwargs that initialized the Mapping
         self.kwargs = {**mapping_kwargs, **self.kwargs}
 
@@ -179,9 +216,36 @@ class Mapping():
     def process_kwargs(self, mapping):
         ''' Apply kwargs in the order they are provided. kwargs are ordered as of python 3.6
         '''
+
+        if 'as_is' in self.kwargs:
+            as_is = self.kwargs['as_is']
+            if as_is:
+                appropriate_setting = "as-written"
+            else:
+                appropriate_setting = "apply-longest-first"
+
+            self.kwargs["rule_ordering"] = appropriate_setting
+            del self.kwargs["as_is"]
+
+            LOGGER.warning(
+                f"mapping from {self.kwargs.get('in_lang')} to {self.kwargs.get('out_lang')} "
+                'is using the deprecated parameter "as_is"; '
+                f"replace `as_is: {as_is}` with `rule_ordering: {appropriate_setting}`"
+            )
+
         # Add defaults
-        if 'as_is' not in self.kwargs:
-            self.kwargs['as_is'] = True
+        if 'rule_ordering' in self.kwargs:
+            # right now, "rule-ordering" is a more explict alias of the "as-is" option.
+            ordering = self.kwargs["rule_ordering"]
+            if ordering not in ("as-written", "apply-longest-first"):
+                LOGGER.error(
+                    f"mapping from {self.kwargs.get('in_lang')} to {self.kwargs.get('out_lang')} "
+                    f"has invalid value '{ordering}' for rule_ordering parameter; "
+                    "rule_ordering must be one of "
+                    '"as-written" or "apply-longest-first"'
+                )
+        else:
+            self.kwargs["rule_ordering"] = "as-written"
         if 'case_sensitive' not in self.kwargs:
             self.kwargs['case_sensitive'] = True
         if 'escape_special' not in self.kwargs:
@@ -196,9 +260,10 @@ class Mapping():
             self.kwargs['in_lang'] = 'und'
         if 'out_lang' not in self.kwargs:
             self.kwargs['out_lang'] = 'und'
+
         # Process kwargs in order received
         for kwarg, val in self.kwargs.items():
-            if kwarg == 'as_is' and not val:
+            if kwarg == 'rule_ordering' and self.wants_rules_sorted():
                 # sort by reverse len
                 mapping = sorted(mapping, key=lambda x: len(
                     x["in"]), reverse=True)
@@ -222,6 +287,14 @@ class Mapping():
         self.processed = True
         return mapping
 
+    def wants_rules_sorted(self) -> bool:
+        """Returns whether the rules will be sorted prior to finalizing the mapping.
+
+        Returns:
+            bool: True if the rules should be sorted.
+        """
+        return self.kwargs['rule_ordering'] == 'apply-longest-first'
+
     def rule_to_regex(self, rule: dict) -> Pattern:
         """Turns an input string (and the context) from an input/output pair
         into a regular expression pattern"
@@ -278,8 +351,24 @@ class Mapping():
         '''
         for io in mapping:
             io['in'], io['out'] = io['out'], io['in']
+            del io['context_before']
+            del io['context_after']
         return mapping
 
+    def extend(self, mapping):
+        """Add all the rules from mapping into self, effectively merging two mappings
+
+        Caveat: if self and mapping have contradictory rules, which one will
+        "win" is unspecified, and may depend on mapping configuration options.
+        """
+        self.mapping.extend(mapping.mapping)
+
+    def deduplicate(self):
+        """Remove duplicate rules found in self, keeping the first copy found."""
+        # Since Python 3.6, dict keeps its element in insertion order (while
+        # set does not), so deduplicating the rules is a one-liner:
+        self.mapping = list({repr(rule): rule for rule in self.mapping}.values())
+
     def add_abbreviations(self, abbs, mappings):
         ''' Return abbreviated forms, given a list of abbreviations.
 
@@ -307,7 +396,7 @@ class Mapping():
                     for io in self.mapping]
         if file_type == 'json':
             with open(fn, 'w', encoding='utf8') as f:
-                json.dump(filtered, f, indent=4)
+                json.dump(filtered, f, indent=4, ensure_ascii=False)
         elif file_type == 'csv':
             with open(fn, 'w', encoding='utf8') as f:
                 writer = csv.DictWriter(f, fieldnames=fieldnames)
@@ -320,12 +409,12 @@ class Mapping():
         ''' Write config to file
         '''
         add_config = False
+        if os.path.isdir(output_path):
+            output_path = os.path.join(output_path, 'config.yaml')
         if os.path.exists(output_path) and os.path.isfile(output_path):
             LOGGER.warning(f'Adding mapping config to file at {output_path}')
             fn = output_path
             add_config = True
-        elif os.path.isdir(output_path):
-            fn = os.path.join(output_path, 'config.yaml')
         else:
             LOGGER.warning(f'writing mapping config to file at {output_path}')
             fn = output_path
@@ -336,7 +425,8 @@ class Mapping():
                 "in_lang": self.kwargs.get('in_lang', 'und'),
                 "out_lang": self.kwargs.get('out_lang', 'und'),
                 "authors": self.kwargs.get('authors', [f'Generated {dt.datetime.now()}']),
-                "as_is": self.kwargs.get('as_is', True),
+                "rule_ordering": self.kwargs.get("rule_ordering", "as-written"),
+                "prevent_feeding": self.kwargs.get('prevent_feeding', False),
                 "case_sensitive": self.kwargs.get('case_sensitive', True),
                 "escape_special": self.kwargs.get('escape_special', False),
                 "norm_form": self.kwargs.get('norm_form', "NFD"),
@@ -355,7 +445,7 @@ class Mapping():
                     existing_data['mappings'][i]['authors'] = template['mappings'][0]['authors']
                     updated = True
                     break
-            if not updated:  
+            if not updated:
                 existing_data['mappings'].append(template['mappings'][0])
             template=existing_data
         with open(fn, 'w', encoding='utf8') as f:
