@@ -3,22 +3,18 @@
 Utilities used by other classes
 
 """
-import os
 import csv
-from collections import defaultdict
-from typing import List
-import regex as re
 import json
+import os
+import unicodedata as ud
+from collections import defaultdict
 from copy import deepcopy
 from pathlib import Path
-import datetime as dt
-import yaml
-
-import unicodedata as ud
-
-from openpyxl import load_workbook
 from typing import Dict
 
+import regex as re
+import yaml
+
 from g2p import exceptions
 from g2p.log import LOGGER
 from g2p.mappings import langs
@@ -62,7 +58,8 @@ def normalize(inp: str, norm_form: str):
         normalized = ud.normalize(norm_form, unicode_escape(inp))
         if normalized != inp:
             LOGGER.debug(
-                'The string %s was normalized to %s using the %s standard and by decoding any Unicode escapes. Note that this is not necessarily the final stage of normalization.',
+                'The string %s was normalized to %s using the %s standard and by decoding any Unicode escapes. '
+                'Note that this is not necessarily the final stage of normalization.',
                 inp, normalized, norm_form)
         return normalized
 
@@ -110,6 +107,7 @@ def pattern_to_fixed_width_lookbehinds(match):
 def load_from_workbook(language):
     ''' Parse mapping from Excel workbook
     '''
+    from openpyxl import load_workbook  # Expensive import, do it only when needed
     work_book = load_workbook(language)
     work_sheet = work_book.active
     # Create wordlist
@@ -161,6 +159,13 @@ def load_from_csv(language, delimiter=','):
     for entry in work_sheet:
         new_io = {"in": "", "out": "",
                   "context_before": "", "context_after": ""}
+        if len(entry) == 0:
+            # Just ignore empty lines in the CSV file
+            continue
+
+        if len(entry) == 1:
+            raise exceptions.MalformedMapping('Entry {} in mapping {} has no "out" value.'.format(entry, language))
+
         new_io['in'] = entry[0]
         new_io['out'] = entry[1]
         try:
@@ -172,7 +177,7 @@ def load_from_csv(language, delimiter=','):
         except IndexError:
             new_io['context_after'] = ''
         for k in new_io:
-            if isinstance(new_io[k], float) or isinstance(new_io[k], int):
+            if isinstance(new_io[k], (float, int)):
                 new_io[k] = str(new_io[k])
         mapping.append(new_io)
 
@@ -194,7 +199,7 @@ def load_from_file(path: str) -> list:
             mapping = json.load(f)
     else:
         raise TypeError(f"Path at {path} is not a valid filetype.")
-    return validate(mapping)
+    return validate(mapping, path)
 
 def load_mapping_from_path(path_to_mapping_config, index=0):
     ''' Loads a mapping from a path, if there is more than one mapping, then it loads based on the int
@@ -209,23 +214,39 @@ def load_mapping_from_path(path_to_mapping_config, index=0):
         # If more than one mapping in the mapping config
         if 'mappings' in mapping:
             try:
-                LOGGER.debug('Loading mapping from %s between "%s" and "%s" at index %s', path_to_mapping_config,
-                            mapping['mappings'][index].get('in_lang', 'und'), mapping['mappings'][index].get('out_lang', 'und'), index)
+                LOGGER.debug(
+                    'Loading mapping from %s between "%s" and "%s" at index %s',
+                    path_to_mapping_config,
+                    mapping['mappings'][index].get('in_lang', 'und'),
+                    mapping['mappings'][index].get('out_lang', 'und'),
+                    index
+                )
                 mapping = mapping['mappings'][index]
             except KeyError:
                 LOGGER.warning(
-                    'An index of %s was provided for the mapping %s but that index does not exist in the mapping. Please check your mapping.', index, path_to_mapping_config)
+                    'An index of %s was provided for the mapping %s but that index does not exist in the mapping. '
+                    'Please check your mapping.',
+                    index,
+                    path_to_mapping_config
+                )
         # Log the warning if an Index other than 0 was provided for a mapping config with a single mapping.
         elif index != 0:
             LOGGER.warning(
-                'An index of %s was provided for the mapping %s but that index does not exist in the mapping. Please check your mapping.', index, path_to_mapping_config)
+                'An index of %s was provided for the mapping %s but that index does not exist in the mapping. '
+                'Please check your mapping.',
+                index,
+                path_to_mapping_config
+            )
         # try to load the data from the mapping data file
         if 'mapping' in mapping:
             mapping['mapping_data'] = load_from_file(
                 os.path.join(path.parent, mapping['mapping']))
+        elif mapping.get("type", "") == "unidecode":
+            # This mapping is not implemented as a regular mapping, but as custom software
+            pass
         else:
             # Is "mapping" key missing?
-            raise exceptions.MalformedMapping
+            raise exceptions.MalformedMapping('Key "mapping:" missing from a mapping in {}.'.format(path))
         # load any abbreviations
         if 'abbreviations' in mapping:
             mapping['abbreviations_data'] = load_abbreviations_from_file(
@@ -244,7 +265,7 @@ def find_mapping(in_lang: str, out_lang: str) -> list:
             return deepcopy(mapping)
     raise exceptions.MappingMissing(in_lang, out_lang)
 
-def validate(mapping):
+def validate(mapping, path):
     try:
         for io in mapping:
             if not 'context_before' in io:
@@ -254,11 +275,12 @@ def validate(mapping):
         valid = all('in' in d for d in mapping) and all(
             'out' in d for d in mapping)
         if not valid:
-            raise exceptions.MalformedMapping()
+            raise exceptions.MalformedMapping('Missing "in" or "out" in an entry in {}.'.format(path))
         return mapping
-    except TypeError:
-        # The JSON probably is not just a list (ie could be legacy readalongs format) TODO: proper exception handling
-        raise exceptions.MalformedMapping()
+    except TypeError as e:
+        # The JSON probably is not just a list (ie could be legacy readalongs format)
+        # TODO: proper exception handling
+        raise exceptions.MalformedMapping('Formatting error in mapping in {}.'.format(path)) from e
 
 def escape_special_characters(to_escape: Dict[str, str]) -> Dict[str, str]:
     for k, v in to_escape.items():
@@ -268,7 +290,9 @@ def escape_special_characters(to_escape: Dict[str, str]) -> Dict[str, str]:
             escaped = v
         if escaped != v:
             LOGGER.debug(
-                f"Escaped special characters in '{v}' with '{escaped}''. Set 'escape_special' to False in your Mapping configuration to disable this.")
+                f"Escaped special characters in '{v}' with '{escaped}''. Set 'escape_special' "
+                "to False in your Mapping configuration to disable this."
+            )
         to_escape[k] = escaped
     return to_escape
 
@@ -308,7 +332,68 @@ def is_dummy(lang: str) -> bool:
 
 class IndentDumper(yaml.Dumper):
     def increase_indent(self, flow=False, indentless=False):
-        return super(IndentDumper, self).increase_indent(flow, False)
+        return super().increase_indent(flow, False)
 
     def ignore_aliases(self, *args):
         return True
+
+
+def merge_if_same_label(lst_of_dicts, text_key, label_key):
+    results = []
+    current_item = None
+    for dct in lst_of_dicts:
+        if label_key not in dct:
+            dct[label_key] = None
+        if not current_item:
+            current_item = deepcopy(dct)
+        elif dct[label_key] == current_item[label_key]:
+            current_item[text_key] += dct[text_key]
+        else:
+            results.append(current_item)
+            current_item = deepcopy(dct)
+    if current_item:
+        results.append(current_item)
+    return results
+
+
+CATEGORIES = {
+    "Cc": "other",  # Other, Control
+    "Cf": "other",  # Other, Format
+    "Cn": "other",  # Other, Not Assigned (no characters in the file have this property)
+    "Co": "letter",  # Other, Private Use
+    "Cs": "other",  # Other, Surrogate
+    "LC": "letter",  # Letter, Cased
+    "Ll": "letter",  # Letter, Lowercase
+    "Lm": "letter",  # Letter, Modifier
+    "Lo": "letter",  # Letter, Other
+    "Lt": "letter",  # Letter, Titlecase
+    "Lu": "letter",  # Letter, Uppercase
+    "Mc": "diacritic",  # Mark, Spacing Combining
+    "Me": "diacritic",  # Mark, Enclosing
+    "Mn": "diacritic",  # Mark, Nonspacing
+    "Nd": "number",  # Number, Decimal Digit
+    "Nl": "number",  # Number, Letter
+    "No": "number",  # Number, Other
+    "Pc": "punctuation",  # Punctuation, Connector
+    "Pd": "punctuation",  # Punctuation, Dash
+    "Pe": "punctuation",  # Punctuation, Close
+    "Pf": "punctuation",  # Punctuation, Final quote (may behave like Ps or Pe depending on usage)
+    "Pi": "punctuation",  # Punctuation, Initial quote (may behave like Ps or Pe depending on usage)
+    "Po": "punctuation",  # Punctuation, Other
+    "Ps": "punctuation",  # Punctuation, Open
+    "Sc": "symbol",  # Symbol, Currency
+    "Sk": "symbol",  # Symbol, Modifier
+    "Sm": "symbol",  # Symbol, Math
+    "So": "symbol",  # Symbol, Other
+    "Zl": "whitespace",  # Separator, Line
+    "Zp": "whitespace",  # Separator, Paragraph
+    "Zs": "whitespace",  # Separator, Space
+}
+
+
+def get_unicode_category(c):
+    """ Maps a character to one of [ "letter", "number", "diacritic", "punctuation",
+        "symbol", "whitespace", "other"] """
+    cat = ud.category(c)
+    assert cat in CATEGORIES
+    return CATEGORIES[cat]
