@@ -1,24 +1,34 @@
+#!/usr/bin/env python3
+
 """ Organize tests into Test Suites
 """
 
 import os
 import sys
-from unittest import TestLoader, TextTestRunner, TestSuite
+from unittest import TestLoader, TestSuite, TextTestRunner
 
 # Unit tests
 from g2p.log import LOGGER
+from g2p.tests.test_api_resources import ResourceIntegrationTest
+from g2p.tests.test_check_ipa_arpabet import CheckIpaArpabetTest
+from g2p.tests.test_cli import CliTest
 from g2p.tests.test_create_mapping import MappingCreationTest
-from g2p.tests.test_mappings import MappingTest
-from g2p.tests.test_network import NetworkTest
+from g2p.tests.test_doctor import DoctorTest
+from g2p.tests.test_fallback import FallbackTest
 from g2p.tests.test_indices import IndicesTest
 from g2p.tests.test_langs import LangTest
-from g2p.tests.test_transducer import TransducerTest
-from g2p.tests.test_cli import CliTest
-from g2p.tests.test_utils import UtilsTest
-from g2p.tests.test_fallback import FallbackTest
-from g2p.tests.test_api_resources import ResourceIntegrationTest
+from g2p.tests.test_mappings import MappingTest
+from g2p.tests.test_network import NetworkTest
 from g2p.tests.test_studio import StudioTest
+from g2p.tests.test_tokenize_and_map import TokenizeAndMapTest
+from g2p.tests.test_tokenizer import TokenizerTest
+from g2p.tests.test_transducer import TransducerTest
+from g2p.tests.test_unidecode_transducer import UnidecodeTransducerTest
+from g2p.tests.test_utils import UtilsTest
+from g2p.tests.test_z_local_config import LocalConfigTest
 
+# Deliberately left out:
+# from g2p.tests.test_doctor_expensive import ExpensiveDoctorTest
 
 LOADER = TestLoader()
 
@@ -26,7 +36,8 @@ TRANSDUCER_TESTS = [
     LOADER.loadTestsFromTestCase(test)
     for test in [
         IndicesTest,
-        TransducerTest
+        TransducerTest,
+        UnidecodeTransducerTest,
     ]
 ]
 
@@ -37,7 +48,10 @@ MAPPINGS_TESTS = [
         MappingCreationTest,
         MappingTest,
         NetworkTest,
-        UtilsTest
+        UtilsTest,
+        TokenizerTest,
+        TokenizeAndMapTest,
+        CheckIpaArpabetTest,
     ]
 ]
 
@@ -49,11 +63,19 @@ LANGS_TESTS = [
 
 INTEGRATION_TESTS = [
     LOADER.loadTestsFromTestCase(test) for test in [
-        CliTest, ResourceIntegrationTest, StudioTest
+        CliTest, ResourceIntegrationTest, StudioTest, DoctorTest
     ]
 ]
 
-DEV_TESTS = TRANSDUCER_TESTS + MAPPINGS_TESTS + LANGS_TESTS + INTEGRATION_TESTS
+# LocalConfigTest has to get run last, to avoid interactions with other test
+# cases, since it has side effects on the global database
+LAST_DEV_TEST = [
+    LOADER.loadTestsFromTestCase(test) for test in [
+        LocalConfigTest,
+    ]
+]
+
+DEV_TESTS = TRANSDUCER_TESTS + MAPPINGS_TESTS + LANGS_TESTS + INTEGRATION_TESTS + LAST_DEV_TEST
 
 
 def run_tests(suite):
@@ -61,13 +83,13 @@ def run_tests(suite):
     '''
     if suite == 'all':
         suite = LOADER.discover(os.path.dirname(__file__))
-    if suite == 'trans':
+    elif suite == 'trans':
         suite = TestSuite(TRANSDUCER_TESTS)
-    if suite == 'langs':
+    elif suite == 'langs':
         suite = TestSuite(LANGS_TESTS)
-    if suite == 'mappings':
+    elif suite == 'mappings':
         suite = TestSuite(MAPPINGS_TESTS)
-    if suite == 'integ':
+    elif suite == 'integ':
         suite = TestSuite(INTEGRATION_TESTS)
     elif suite == 'dev':
         suite = TestSuite(DEV_TESTS)
@@ -76,7 +98,7 @@ def run_tests(suite):
         LOGGER.error("Please specify a test suite to run: i.e. 'dev' or 'all'")
     else:
         return runner.run(suite)
-    
+
 
 
 if __name__ == "__main__":
