@@ -6,17 +6,20 @@ which are responsible for performing transductions in the g2p library.
 
 import re
 import copy
+import text_unidecode
 from typing import Dict, List, Pattern, Tuple, Union
 from collections import defaultdict, OrderedDict
 from collections.abc import Iterable
 from g2p.mappings import Mapping
-from g2p.mappings.utils import create_fixed_width_lookbehind, normalize
+from g2p.mappings.tokenizer import DefaultTokenizer
+from g2p.mappings.utils import create_fixed_width_lookbehind, normalize, is_ipa
+from g2p.mappings.langs.utils import is_arpabet, is_panphon
 from g2p.exceptions import MalformedMapping
 from g2p.log import LOGGER
 
 # Avoid TypeError in Python < 3.7 (see
 # https://stackoverflow.com/questions/6279305/typeerror-cannot-deepcopy-this-pattern-object)
-copy._deepcopy_dispatch[type(re.compile(''))] = lambda r, _: r
+copy._deepcopy_dispatch[type(re.compile(""))] = lambda r, _: r
 
 # An Index is typed as follows:
 # {input_index: int, {'input_string': str, 'output': {output_index: int, str}}}
@@ -32,7 +35,7 @@ Index = Dict
 ChangeLog = List[List[int]]
 
 
-class TransductionGraph():
+class TransductionGraph:
     """This is the object returned after performing a transduction using a Transducer.
 
     Each TransductionGraph must be initialized with an input string.
@@ -61,7 +64,8 @@ class TransductionGraph():
     @input_string.setter
     def input_string(self, value):
         raise ValueError(
-            f'Sorry, you tried to change the input string to {value} but it cannot be changed')
+            f"Sorry, you tried to change the input string to {value} but it cannot be changed"
+        )
 
     @property
     def output_string(self):
@@ -81,7 +85,8 @@ class TransductionGraph():
     @input_nodes.setter
     def input_nodes(self, value):
         raise ValueError(
-            f'Sorry, you tried to change the input nodes to {value} but they cannot be changed.')
+            f"Sorry, you tried to change the input nodes to {value} but they cannot be changed."
+        )
 
     @property
     def output_nodes(self):
@@ -91,7 +96,8 @@ class TransductionGraph():
     @output_nodes.setter
     def output_nodes(self, value):
         raise ValueError(
-            f'Sorry, you tried to change the output nodes to {value} but they cannot be changed directly. Change output_string instead.')
+            f"Sorry, you tried to change the output nodes to {value} but they cannot be changed directly. Change output_string instead."
+        )
 
     @property
     def edges(self):
@@ -104,7 +110,7 @@ class TransductionGraph():
 
     @property
     def debugger(self):
-        """List[dict]: A list of rules applied during the transformation. Useful for debugging."""
+        """List[dict]: A list of lists of rules applied during the transformation. Useful for debugging."""
         return self._debugger
 
     @debugger.setter
@@ -119,21 +125,60 @@ class TransductionGraph():
     @tiers.setter
     def tiers(self, value):
         raise ValueError(
-            f'Sorry, you tried to change the tiers to {value} but they cannot be changed')
+            f"Sorry, you tried to change the tiers to {value} but they cannot be changed"
+        )
 
     def pretty_edges(self):
         edges = copy.deepcopy(self._edges)
         edges.sort(key=lambda x: x[0])
         for i, edge in enumerate(edges):
-            if edge[1] != None:
-                edges[i] = [self._input_nodes[edge[0]]
-                            [1], self._output_nodes[edge[1]][1]]
-            else:
+            if edge[1] is None:
                 edges[i] = [self._input_nodes[edge[0]][1], None]
+            else:
+                edges[i] = [
+                    self._input_nodes[edge[0]][1],
+                    self._output_nodes[edge[1]][1],
+                ]
         return edges
 
+    def as_dict(self):
+        return {
+            "edges": self._edges,
+            "input": self._input_string,
+            "output": self._output_string,
+            "input_nodes": self._input_nodes,
+            "output_nodes": self._output_nodes,
+        }
 
-class Transducer():
+    def clear_debugger(self):
+        self._debugger = []
+
+    def append(self, tg):
+        """ Append the nodes, edges, strings and debugger from tg to self,
+            shifting indices so tg nodes and edges are added after those of self.
+        """
+        in_offset = len(self._input_nodes)
+        out_offset = len(self._output_nodes)
+        # append input and output strings
+        self._input_string += tg._input_string
+        self._output_string += tg._output_string
+        # append nodes
+        self._input_nodes += [(i + in_offset, x) for (i, x) in tg._input_nodes]
+        self._output_nodes += [(i + out_offset, x) for (i, x) in tg._output_nodes]
+        # append edges
+        self._edges += [
+            (i + in_offset, None if j is None else j + out_offset)
+            for (i, j) in tg._edges
+        ]
+        # append debuggers
+        self._debugger += tg._debugger
+
+    def __iadd__(self, tg):
+        self.append(tg)
+        return self
+
+
+class Transducer:
     """This is the fundamental class for performing conversions in the g2p library.
 
     Each Transducer must be initialized with a Mapping object. The Transducer object can then be called to apply the rules from Mapping on a given input.
@@ -144,11 +189,11 @@ class Transducer():
 
     def __init__(self, mapping: Mapping):
         self.mapping = mapping
-        self.case_sensitive = mapping.kwargs['case_sensitive']
-        self.norm_form = mapping.kwargs.get('norm_form', 'none')
-        self.out_delimiter = mapping.kwargs.get('out_delimiter', '')
-        self._index_match_pattern = re.compile(r'(?<={)\d+(?=})')
-        self._char_match_pattern = re.compile(r'[^0-9\{\}]+(?={\d+})', re.U)
+        self.case_sensitive = mapping.kwargs["case_sensitive"]
+        self.norm_form = mapping.kwargs.get("norm_form", "none")
+        self.out_delimiter = mapping.kwargs.get("out_delimiter", "")
+        self._index_match_pattern = re.compile(r"(?<={)\d+(?=})")
+        self._char_match_pattern = re.compile(r"[^0-9\{\}]+(?={\d+})", re.U)
 
     def __repr__(self):
         return f"{self.__class__} between {self.mapping.kwargs.get('in_lang', 'und')} and {self.mapping.kwargs.get('out_lang', 'und')}"
@@ -179,31 +224,34 @@ class Transducer():
             intermediate_ord = ord(string[0])
             return intermediate_ord - 983040
         else:
-            return - 1
+            return -1
 
     def resolve_intermediate_chars(self, output_string):
-        ''' Go through all chars and resolve any intermediate characters from the Private Supplementary Use Area
+        """ Go through all chars and resolve any intermediate characters from the Private Supplementary Use Area
             to their mapped equivalents.
-        '''
+        """
         indices_seen = defaultdict(int)
         for i, char in enumerate(output_string):
             intermediate_index = self._pua_to_index(char)
             # if not Private Supplementary Use character
             if intermediate_index < 0:
                 continue
-            else:
-                output_char_index = indices_seen[intermediate_index]
-                try:
-                    output_string = output_string[:i] + \
-                        self.mapping[intermediate_index]['out'][output_char_index] + \
-                        output_string[i+1:]
-                except IndexError:
-                    indices_seen[intermediate_index] = 0
-                    output_char_index = 0
-                    output_string = output_string[:i] + \
-                        self.mapping[intermediate_index]['out'][output_char_index] + \
-                        output_string[i+1:]
-                indices_seen[intermediate_index] += 1
+            output_char_index = indices_seen[intermediate_index]
+            try:
+                output_string = (
+                    output_string[:i]
+                    + self.mapping[intermediate_index]["out"][output_char_index]
+                    + output_string[i + 1 :]
+                )
+            except IndexError:
+                indices_seen[intermediate_index] = 0
+                output_char_index = 0
+                output_string = (
+                    output_string[:i]
+                    + self.mapping[intermediate_index]["out"][output_char_index]
+                    + output_string[i + 1 :]
+                )
+            indices_seen[intermediate_index] += 1
         return output_string
 
     def update_explicit_indices(self, tg, match, io, intermediate_diff, out_string):
@@ -216,34 +264,38 @@ class Transducer():
                 For this, the Mapping could be given the following: [{'in': 'k{1}\u0313{2}', 'out': "'{2}k{1}"}]
                 Indices are found with r'(?<={)\d+(?=})' and characters are found with r'[^0-9\{\}]+(?={\d+})'
         """
-        input_char_matches = [x.group()
-                              for x in self._char_match_pattern.finditer(io['in'])]
+        input_char_matches = [
+            x.group() for x in self._char_match_pattern.finditer(io["in"])
+        ]
         input_match_indices = [
-            x.group() for x in self._index_match_pattern.finditer(io['in'])]
+            x.group() for x in self._index_match_pattern.finditer(io["in"])
+        ]
         inputs = {}
         index = 0
         start = match.start() + intermediate_diff
         for i, m in enumerate(input_match_indices):
             for j, char in enumerate(input_char_matches[i]):
                 if m in inputs:
-                    inputs[m].append({'index': index + start, 'string': char})
+                    inputs[m].append({"index": index + start, "string": char})
                 else:
-                    inputs[m] = [{'index': index + start, 'string': char}]
+                    inputs[m] = [{"index": index + start, "string": char}]
                 index += 1
         output_char_matches = [
-            x.group() for x in self._char_match_pattern.finditer(out_string)]
+            x.group() for x in self._char_match_pattern.finditer(out_string)
+        ]
         output_match_indices = [
-            x.group() for x in self._index_match_pattern.finditer(out_string)]
+            x.group() for x in self._index_match_pattern.finditer(out_string)
+        ]
         outputs = {}
         index = 0
         for i, m in enumerate(output_match_indices):
             for j, char in enumerate(output_char_matches[i]):
                 if m in outputs:
-                    outputs[m].append({'index': index + start, 'string': char})
+                    outputs[m].append({"index": index + start, "string": char})
                 else:
-                    outputs[m] = [{'index': index + start, 'string': char}]
+                    outputs[m] = [{"index": index + start, "string": char}]
                 index += 1
-        out_string = re.sub(re.compile(r'{\d+}'), '', out_string)
+        out_string = re.sub(re.compile(r"{\d+}"), "", out_string)
         deleted = 0
         for match_index, input_matches in inputs.items():
             try:
@@ -253,52 +305,57 @@ class Transducer():
             if len(input_matches) == len(output_matches):
                 shortest = input_matches
                 longest = output_matches
-                process = 'basic'
+                process = "basic"
             elif len(input_matches) < len(output_matches):
                 shortest = input_matches
                 longest = output_matches
-                process = 'insert'
+                process = "insert"
             else:
                 shortest = output_matches
                 longest = input_matches
-                process = 'delete'
+                process = "delete"
             for i, char in enumerate(longest):
-                if process == 'basic' or i <= len(shortest) - 1:
-                    input_index = input_matches[i]['index'] - intermediate_diff
-                    output_index = output_matches[i]['index']
+                if process == "basic" or i <= len(shortest) - 1:
+                    input_index = input_matches[i]["index"] - intermediate_diff
+                    output_index = output_matches[i]["index"]
                     # don't allow insertion in basic process
                     if output_index >= len(match.group()) + start:
                         output_index = len(match.group()) + start - 1
-                    tg.output_string = tg.output_string[:output_index] + \
-                        char['string'] + tg.output_string[output_index+1:]
+                    tg.output_string = (
+                        tg.output_string[:output_index]
+                        + char["string"]
+                        + tg.output_string[output_index + 1 :]
+                    )
                     tg.edges = [x for x in tg.edges if x[1] != output_index]
                     tg.edges.append([input_index, output_index])
                 else:
-                    if process == 'insert':
-                        input_index = input_matches[-1]['index'] - \
-                            intermediate_diff
-                        output_index = output_matches[i]['index']
-                        tg.output_string = tg.output_string[:output_index] + \
-                            char['string'] + tg.output_string[output_index:]
+                    if process == "insert":
+                        input_index = input_matches[-1]["index"] - intermediate_diff
+                        output_index = output_matches[i]["index"]
+                        tg.output_string = (
+                            tg.output_string[:output_index]
+                            + char["string"]
+                            + tg.output_string[output_index:]
+                        )
                         for i, edge in enumerate(tg.edges):
                             if edge[1] != None and edge[1] >= output_index:
                                 tg.edges[i][1] += 1
                         tg.edges.append([input_index, output_index])
                     else:
-                        input_index = input_matches[i]['index'] - \
-                            intermediate_diff
+                        input_index = input_matches[i]["index"] - intermediate_diff
                         if output_matches:
-                            output_index = output_matches[-1]['index'] - deleted
+                            output_index = output_matches[-1]["index"] - deleted
                         else:
                             output_index = input_index + intermediate_diff - deleted
-                        tg.output_string = tg.output_string[:output_index] + \
-                            tg.output_string[output_index + 1:]
+                        tg.output_string = (
+                            tg.output_string[:output_index]
+                            + tg.output_string[output_index + 1 :]
+                        )
                         deleted += 1
                         if len(output_matches) > 0:
                             if [input_index, output_index] not in tg.edges:
                                 tg.edges.append([input_index, output_index])
-                            tg.edges = [
-                                x for x in tg.edges if x[1] != output_index]
+                            tg.edges = [x for x in tg.edges if x[1] != output_index]
                         else:
                             for i, edge in enumerate(tg.edges):
                                 if edge[1] != None and edge[1] == output_index:
@@ -314,19 +371,22 @@ class Transducer():
         out_length = len(out_string)
         if in_length == out_length:
             for i, char in enumerate(out_string):
-                tg.output_string = tg.output_string[:i +
-                                                    start] + char + tg.output_string[i+start+1:]
+                tg.output_string = (
+                    tg.output_string[: i + start]
+                    + char
+                    + tg.output_string[i + start + 1 :]
+                )
             return
         # default insertion(s)
         elif in_length < out_length:
             longest = out_string
             shortest = in_string
-            process = 'insert'
+            process = "insert"
         # default deletion(s)
         else:
             longest = in_string
             shortest = out_string
-            process = 'delete'
+            process = "delete"
         # iterate the longest string
         deleted = 0
         last_input_node = start
@@ -334,19 +394,25 @@ class Transducer():
         for i, char in enumerate(longest):
             # if the shorter string still has that output, keep that index
             if i <= len(shortest) - 1:
-                tg.output_string = tg.output_string[:i+start] + \
-                    out_string[i] + tg.output_string[i+start+1:]
+                tg.output_string = (
+                    tg.output_string[: i + start]
+                    + out_string[i]
+                    + tg.output_string[i + start + 1 :]
+                )
                 last_input_node = i + start
                 last_output_node = i + start
             # otherwise...
             else:
                 # add a new node and increment each following node
                 # log the change in order to update the edges.
-                if process == 'insert':
+                if process == "insert":
                     # Nodes
                     index_to_add = i + start
-                    tg.output_string = tg.output_string[:index_to_add] + \
-                        char + tg.output_string[index_to_add:]
+                    tg.output_string = (
+                        tg.output_string[:index_to_add]
+                        + char
+                        + tg.output_string[index_to_add:]
+                    )
                     # Edges
                     # Remove previously deleted and increment
                     for i, edge in enumerate(tg.edges):
@@ -354,7 +420,8 @@ class Transducer():
                             tg.edges[i][1] += 1
                     # add edge to index of last input character
                     last_input_node = max(
-                        [x[0] for x in tg.edges if x[1] == last_output_node])
+                        [x[0] for x in tg.edges if x[1] == last_output_node]
+                    )
                     last_output_node = index_to_add
                     tg.edges.append([last_input_node, index_to_add])
                 # delete the node and decrement each following node
@@ -362,21 +429,22 @@ class Transducer():
                 else:
                     # Nodes
                     index_to_delete = i + start - deleted
-                    tg.output_string = tg.output_string[:index_to_delete] + \
-                        tg.output_string[index_to_delete + 1:]
+                    tg.output_string = (
+                        tg.output_string[:index_to_delete]
+                        + tg.output_string[index_to_delete + 1 :]
+                    )
                     deleted += 1
                     # Edges
                     # delete
                     last_input_node = max(
-                        [x[0] for x in tg.edges if x[1] == index_to_delete])
+                        [x[0] for x in tg.edges if x[1] == index_to_delete]
+                    )
                     # if rule is not just a simple deletion,
                     # add an edge between the node and the last output node
                     if out_length > 0:
                         if [last_input_node, last_output_node] not in tg.edges:
-                            tg.edges.append(
-                                [last_input_node, last_output_node])
-                        tg.edges = [x for x in tg.edges if x[1]
-                                    != index_to_delete]
+                            tg.edges.append([last_input_node, last_output_node])
+                        tg.edges = [x for x in tg.edges if x[1] != index_to_delete]
                     else:
                         for i, edge in enumerate(tg.edges):
                             if edge[1] != None and edge[1] == index_to_delete:
@@ -386,56 +454,126 @@ class Transducer():
                         if edge[1] != None and edge[1] > index_to_delete:
                             tg.edges[i][1] -= 1
 
+    def apply_unidecode(self, to_convert: str):
+        if self.norm_form:
+            to_convert = normalize(to_convert, self.norm_form)
+        tg = TransductionGraph(to_convert)
+
+        # Conversion is done character by character using unidecode
+        converted = [
+            text_unidecode.unidecode(c)
+            for c in to_convert
+        ]
+        tg.output_string = "".join(converted)
+
+        # Edges are calculated to follow the conversion step by step
+        if tg.output_string == "":
+            # Some inputs get completely deleted by unidecode, in which case there are no
+            # valid edges to output.
+            tg.edges = []
+        else:
+            edges = []
+            x_len, y_len = 0, 0
+            for tgt in converted:
+                if tgt:
+                    for c in tgt:
+                        edges.append((x_len, y_len))
+                        y_len += 1
+                else:
+                    edges.append((x_len, max(y_len-1, 0)))
+                x_len += 1
+            tg.edges = edges
+
+        return tg
+
     def apply_rules(self, to_convert: str):
+        if self.mapping.kwargs.get("type", "") == "unidecode":
+            return self.apply_unidecode(to_convert)
+
         # perform any normalization
         if not self.case_sensitive:
             to_convert = to_convert.lower()
         if self.norm_form:
             to_convert = normalize(to_convert, self.norm_form)
         tg = TransductionGraph(to_convert)
+        tg.debugger.append([])
         # initialize values
         intermediate_forms = False
         # iterate rules
         for io in self.mapping:
             # Do not allow empty rules
-            if not io['in'] and not io['out']:
+            if not io["in"] and not io["out"]:
                 continue
             io = copy.deepcopy(io)
             intermediate_diff = 0
-            for match in io['match_pattern'].finditer(tg.output_string):
+            for match in io["match_pattern"].finditer(tg.output_string):
                 debug_string = tg.output_string
                 start = match.start() + intermediate_diff
                 end = match.end() + intermediate_diff
-                if 'intermediate_form' in io:
-                    out_string = io['intermediate_form']
+                if "intermediate_form" in io:
+                    out_string = io["intermediate_form"]
                     intermediate_forms = True
                 else:
-                    out_string = io['out']
+                    out_string = io["out"]
                 if self.out_delimiter:
-                    # if not end segment, add delimiter
-                    if not end >= len(tg.output_string):
-                        out_string += self.out_delimiter
-                if any(self._char_match_pattern.finditer(io['in'])) and any(self._char_match_pattern.finditer(out_string)):
+                    out_string += self.out_delimiter
+                if any(self._char_match_pattern.finditer(io["in"])) and any(
+                    self._char_match_pattern.finditer(out_string)
+                ):
                     self.update_explicit_indices(
-                        tg, match, io, intermediate_diff, out_string)
+                        tg, match, io, intermediate_diff, out_string
+                    )
                 else:
                     self.update_default_indices(
-                        tg, match, intermediate_diff, out_string)
-                if io['in'] != io['out']:
-                    tg.debugger.append({'input': debug_string,
-                                        'output': tg.output_string,
-                                        'rule': {k: v for k, v in io.items() if k != 'match_pattern'},
-                                        'start': start,
-                                        'end': end})
-                out_string = re.sub(re.compile(r'{\d+}'), '', out_string)
+                        tg, match, intermediate_diff, out_string
+                    )
+                if io["in"] != io["out"]:
+                    tg.debugger[-1].append(
+                        {
+                            "input": debug_string,
+                            "output": tg.output_string,
+                            "rule": {
+                                k: v for k, v in io.items() if k != "match_pattern"
+                            },
+                            "start": start,
+                            "end": end,
+                        }
+                    )
+                out_string = re.sub(re.compile(r"{\d+}"), "", out_string)
                 intermediate_diff += len(out_string) - len(match.group())
         if intermediate_forms:
-            tg.output_string = self.resolve_intermediate_chars(
-                tg.output_string)
-        tg.edges = list(dict.fromkeys(
-            [tuple(x) for x in sorted(tg.edges, key=lambda x: x[0])]))
+            tg.output_string = self.resolve_intermediate_chars(tg.output_string)
+        tg.edges = list(
+            dict.fromkeys([tuple(x) for x in sorted(tg.edges, key=lambda x: x[0])])
+        )
         return tg
 
+    def check(self, tg: TransductionGraph, shallow=False, display_warnings=False, original_input=None):
+        out_lang = self.mapping.kwargs["out_lang"]
+        if out_lang == "eng-arpabet":
+            if not is_arpabet(tg.output_string):
+                if display_warnings:
+                    display_input = original_input if original_input else tg.input_string
+                    LOGGER.warning(
+                        f'Transducer output "{tg.output_string}" for input "{display_input}" is not fully valid eng-arpabet as recognized by soundswallower.'
+                    )
+                return False
+            else:
+                return True
+        elif is_ipa(out_lang):
+            if not is_panphon(tg.output_string, display_warnings=display_warnings):
+                if display_warnings:
+                    display_input = original_input if original_input else tg.input_string
+                    LOGGER.warning(
+                        f'Transducer output "{tg.output_string}" for input "{display_input}" is not fully valid {out_lang}.'
+                    )
+                return False
+            else:
+                return True
+        else:
+            # No check implemented at this tier, just return True
+            return True
+
 
 class CompositeTransductionGraph(TransductionGraph):
     """This is the object returned after performing a transduction using a CompositeTransducer.
@@ -465,7 +603,8 @@ class CompositeTransductionGraph(TransductionGraph):
     @tiers.setter
     def tiers(self, value):
         raise ValueError(
-            f'Sorry, you tried to change the tiers to {value} but they cannot be changed')
+            f"Sorry, you tried to change the tiers to {value} but they cannot be changed"
+        )
 
     def pretty_edges(self):
         pretty_edges = []
@@ -473,16 +612,46 @@ class CompositeTransductionGraph(TransductionGraph):
             edges = copy.deepcopy(edges)
             edges.sort(key=lambda x: x[0])
             for i, edge in enumerate(edges):
-                if edge[1] != None:
-                    edges[i] = [self.tiers[tier_i].input_nodes[edge[0]]
-                                [1], self.tiers[tier_i].output_nodes[edge[1]][1]]
-                else:
+                if edge[1] is None:
                     edges[i] = [self.tiers[tier_i].input_nodes[edge[0]][1], None]
+                else:
+                    edges[i] = [
+                        self.tiers[tier_i].input_nodes[edge[0]][1],
+                        self.tiers[tier_i].output_nodes[edge[1]][1],
+                    ]
             pretty_edges.append(edges)
         return pretty_edges
 
+    def as_dict(self):
+        return {
+            "edges": self._edges,
+            "input": self._input_string,
+            "output": self._output_string,
+            "input_nodes": self._input_nodes,
+            "output_nodes": self._output_nodes,
+        }
 
-class CompositeTransducer():
+    def append(self, tg):
+        if isinstance(tg, CompositeTransductionGraph):
+            assert len(self._tiers) == len(tg._tiers)
+            for i in range(len(self._tiers)):
+                self._tiers[i].append(copy.deepcopy(tg.tiers[i]))
+        else:
+            for tier in self._tiers:
+                tier.append(copy.deepcopy(tg))
+        self.__init__(self.tiers)
+
+    def __iadd__(self, tg):
+        self.append(tg)
+        return self
+
+    def clear_debugger(self):
+        self._debugger = []
+        for tier in self._tiers:
+            tier.clear_debugger()
+
+
+class CompositeTransducer:
     """This class combines Transducer objects to form a CompositeTransducer object.
 
     Attributes:
@@ -491,7 +660,7 @@ class CompositeTransducer():
 
     def __init__(self, transducers: List[Transducer]):
         self._transducers = transducers
-        self._tiers = []
+        self.norm_form = transducers[0].norm_form if transducers else "none"
 
     def __repr__(self):
         return f"{self.__class__} between {self._transducers[0].mapping.kwargs.get('in_lang', 'und')} and {self._transducers[-1].mapping.kwargs.get('out_lang', 'und')}"
@@ -500,8 +669,87 @@ class CompositeTransducer():
         return self.apply_rules(to_convert)
 
     def apply_rules(self, to_convert: str):
+        tg_list = []
         for transducer in self._transducers:
             tg = transducer(to_convert)
-            self._tiers.append(tg)
+            tg_list.append(tg)
             to_convert = tg.output_string
-        return CompositeTransductionGraph(self._tiers)
+        return CompositeTransductionGraph(tg_list)
+
+    def check(
+        self, tg: CompositeTransductionGraph, shallow=False, display_warnings=False
+    ):
+        assert len(self._transducers) == len(tg._tiers)
+        if shallow:
+            return self._transducers[-1].check(
+                tg._tiers[-1],
+                display_warnings=display_warnings,
+                original_input=tg.input_string
+            )
+        else:
+            result = True
+            for i, transducer in enumerate(self._transducers):
+                if not transducer.check(
+                    tg._tiers[i],
+                    display_warnings=display_warnings,
+                    original_input=tg.input_string
+                ):
+                    # Don't short circuit if warnings are required
+                    if display_warnings:
+                        result = False
+                    else:
+                        return False
+            return result
+
+
+class TokenizingTransducer:
+    """This class combines tokenization and transduction.
+
+    Attributes:
+        transducer (Transducer): A Tranducer object for the mapping part
+        tokenizer (DefaultTokenizer): A Tokenizer object to split the string before mapping
+    """
+
+    def __init__(self, transducer: Transducer, tokenizer: DefaultTokenizer):
+        self._transducer = transducer
+        self._tokenizer = tokenizer
+
+    def __call__(self, to_convert: str):
+        # perform normalization before tokenizing, since it can change tokenization
+        if self._transducer.norm_form:
+            to_convert = normalize(to_convert, self._transducer.norm_form)
+
+        # Initialize the transducer on an empty string so we can handle inputs
+        # that start with a non-token correctly.
+        tg = self._transducer("")
+        tg.clear_debugger()  # clear the meaningless initial debugger
+
+        for token in self._tokenizer.tokenize_text(to_convert):
+            if token["is_word"]:
+                word_tg = self._transducer(token["text"])
+                tg += word_tg
+            else:
+                non_word_tg = TransductionGraph(token["text"])
+                tg += non_word_tg
+        return tg
+
+    def check(self, tg: TransductionGraph, shallow=False, display_warnings=False):
+        # The obvious implementation fails, because we need to check only the words, not
+        # the text between the words!
+        # return self._transducer.check(tg) # <- complains about characters between words
+
+        # So, sadly, we redo the work of transduction so we can check the words only, step
+        # by step. I don't like this solution, but I don't see how to get around it.
+        result = True
+        for token in self._tokenizer.tokenize_text(tg.input_string):
+            if token["is_word"] and not self._transducer.check(
+                self._transducer(token["text"]),
+                shallow,
+                display_warnings=display_warnings,
+            ):
+                # Don't short circuit if warnings are required
+                if display_warnings:
+                    result = False
+                else:
+                    return False
+        return result
